@@ -3,6 +3,7 @@
 from attr import attrs, attrib
 import base64
 import concurrent.futures
+import hashlib
 import json
 import queue
 import mimetypes
@@ -55,11 +56,13 @@ def get_request(url, headers, params=None):
 
 
 def handle_patch(patch_path):
+    patch_is_web_url = False
     patch_content = None
     patch_file = os.path.basename(patch_path)
     if urlparse(patch_path).scheme in ["http", "https"]:
         patch_file = patch_path
         patch_content = b""
+        patch_is_web_url = True
     elif os.path.isfile(patch_path):
         file_type, file_encoding = mimetypes.guess_type(patch_path)
         if file_type in [
@@ -96,6 +99,9 @@ def handle_patch(patch_path):
             tar.add(patch_path, arcname=os.path.sep)
         with open(tar_file, "rb") as patch:
             patch_content = base64.b64encode(patch.read())
+    if not patch_is_web_url:
+        patch_sha256 = hashlib.sha256(patch_content).hexdigest()
+        patch_file = f"{patch_sha256}/{patch_file}"
     return (patch_file, patch_content.decode("utf-8"))
 
 
@@ -181,6 +187,7 @@ class Build(HeadersMixin):
     def generate_build_request(self, plan=None):
         """return a build data in a python dict"""
         build_entry = {}
+        patch = {}
         build_entry["client_token"] = self.client_token
         build_entry["git_repo"] = self.git_repo
         if self.git_ref:
@@ -202,9 +209,10 @@ class Build(HeadersMixin):
         if self.patch_series:
             (
                 build_entry["kernel_patch_file"],
-                build_entry["kernel_patch"],
+                kernel_patch,
             ) = handle_patch(self.patch_series)
-        return build_entry
+            patch[build_entry["kernel_patch_file"]] = kernel_patch
+        return (build_entry, patch)
 
     def verify_build_parameters(self):
         """Pre-check the build arguments"""
@@ -222,8 +230,10 @@ class Build(HeadersMixin):
 
     def build(self, plan=None):
         """Submit the build request"""
-        data = []
-        data.append(self.generate_build_request(plan=plan))
+        data = {}
+        build_entry, patch = self.generate_build_request(plan=plan)
+        data["builds"] = [build_entry]
+        data["patches"] = patch
         url = self.tuxapi_url + "/v1/groups/{}/projects/{}/builds".format(
             self.group, self.project
         )
@@ -425,12 +435,14 @@ class BuildSet(HeadersMixin):
     def build(self):
         """Submit the build request"""
 
-        data = []
+        data = {"builds": [], "patches": {}}
         url = self.tuxapi_url + "/v1/groups/{}/projects/{}/builds".format(
             self.group, self.project
         )
         for build_object in self.builds:
-            data.append(build_object.generate_build_request())
+            build_entry, patch = build_object.generate_build_request()
+            data["builds"].append(build_entry)
+            data["patches"].update(patch)
         json_data = post_request(url, self.headers, data)
 
         builds_by_client_token = {b.client_token: b for b in self.builds}
@@ -531,13 +543,14 @@ class Plan(HeadersMixin):
 
         builds_to_submit = [b for b in builds if b]
         if builds_to_submit:
-            ret = post_request(
-                f"{self.url}/builds",
-                self.headers,
-                [b.generate_build_request(plan=self.plan) for b in builds_to_submit],
-            )
+            req_data = {"builds": [], "patches": {}}
+            for b in builds_to_submit:
+                build_entry, patch = b.generate_build_request(plan=self.plan)
+                req_data["builds"].append(build_entry)
+                req_data["patches"].update(patch)
+            ret = post_request(f"{self.url}/builds", self.headers, req_data)
             # Updating builds_to_submit will update values in builds
-            for (build, data) in zip(builds_to_submit, ret):
+            for (build, data) in zip(builds_to_submit, ret["builds"]):
                 build.build_data = data["download_url"]
                 build.uid = data["uid"]
                 build.status = data
